/**
  * Classe de base en version une pour les applications AbulEdu permettant de structurer
  * nos logiciels et respecter ainsi un certain nombre de choses
  *
  * @see https://redmine.ryxeo.com/projects/ryxeo/wiki/AbulEduApplicationV1
  * @author 2011-2012 Eric Seigne <eric.seigne@ryxeo.com>
  * @see The GNU Public License (GPL)
  *
  * #AbulEduApplicationV1
  * include(lib/abuleduapplicationv1/abuleduapplicationv1.pri)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
  * for more details.
  *
  * You should have received a copy of the GNU General Public License along
  * with this program; if not, write to the Free Software Foundation, Inc.,
  * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
  */

#ifndef ABULEDUAPPLICATIONV1_H
#define ABULEDUAPPLICATIONV1_H

#ifndef DEBUG_ABULEDUAPPLICATIONV1
#define DEBUG_ABULEDUAPPLICATIONV1 false
#endif


#include <stdio.h>

#include <QApplication>
#include <QPluginLoader>
#include <QNetworkProxy>
#include <QSharedMemory>
#include <QProxyStyle>
#include <QMessageBox>
#include <QtXml/QtXml>
#include <QSettings>
#include <QHostInfo>
#include <QWidget>
#include <QThread>
#include <QString>
#include <QDebug>
#include <QEvent>
#include <QDate>
#include <QHash>
#include <QDir>
#include <QFontDatabase>

#include "qtsingleapplication/src/qtsingleapplication.h"
#include "abuleduconstantesv1.h"
#include "abuleduproxypassworddialog.h"
#include "abuledunetworkthreadv1.h"
#include "abuledunetworkaccessmanagerv1.h"
#include "abuledumessageboxv1.h"
#include "abuleduidentitesv1.h"
#include "abuleduvirtualkeyboardv1.h"

/**
 * Pour avoir l'acces global a cet objet ... au lieu de qApp on aura
 * abeApp->
 * par contre il faudra penser à faire un #include "abuleduapplicationv1.h"
 * pour ça ...
 */
#ifndef abeApp
#define abeApp ((AbulEduApplicationV1 *) qApp)
#endif

/**
  * @file abuleduapplicationv1.h
  * @version 1
  * @brief L'utilisation de abuleduapplicationv1 permet d'automatiser les différentes choses suivantes:
  * - spécifier le "nom long" des applications à l'aide de setAbeApplicationLongName
  * - vérifier si une nouvelle version du logiciel est disponible en téléchargement
  * - centraliser un connecteur réseau (QNetworkAccessManager) et les détections de proxy et autres authentifications
  * - charger des éventuels plugins
  * - stocker la valeur du ratio de mise à l'échelle des objets graphiques
  * ... et plus encore dès que vous nous donnez des idées :D
  * @link https://redmine.ryxeo.com/projects/ryxeo/wiki/AbulEduApplicationV1
  */

class AbulEduApplicationV1: public QtSingleApplication
{
    Q_OBJECT

public:
    /** Constructeur de la classe
      * @param QString appName, chaine de caractere, le nom de l'application
      * @param QString appVersion, chaine de caractere, la version de l'application
      * @param QString organizationDomain, chaine de caractere, le domaine de l'organisation, initialise par defaut a ""
      * @param QString organizationName, chaine de caractere, le nom de l'organisation, initialise par defaut a ""
      */
    AbulEduApplicationV1(int &argc, char *argv[], QString appName, QString appVersion, QString organizationDomain = "", QString organizationName = "");

    /** Destructeur par defaut de la classe
      */
    ~AbulEduApplicationV1();

    /** Choix du niveau de debug (pour l'instant 1 ou 0)
      * @param int i, entier, le niveau de debug (pour l'instant 1 ou 0)
      * @brief methode virtuelle
      * @return void
      */
    virtual void setAbeApplicationDebugLevel(int i=0);

    /** Permet de récupérer le nom 'long' de l'application
      * @return QString, chaine de caractere, le nom 'long' de l'application
      */
    QString getAbeApplicationLongName();

    /** Permet d'affecter le nom 'long' de l'application, ce qu'on avait en variable globale du temps de abuleduexercicev0
      * remplace extern const QString abeApplicationLongName = QObject::trUtf8("AbulEdu - Livret de compétences");
      * @param QString s, chaine de caractere, le nom 'long' que l'on veut donner à l'application
      * @return void
      */
    void setAbeApplicationLongName(QString s);

    /** Retourne la langue à utiliser dans les interfaces */
    QString getAbeApplicationInterfaceLang();

    /** Enregistre la langue à utiliser dans les interfaces */
    void setAbeApplicationInterfaceLang(QString lang);

    /** Permet d'affecter le ratio de mise a l'echelle des objets graphiques
      * @param float ratio, flottant, le ratio de mise a l'echelle des objets graphiques
      * @return void
      */
    void setAbeApplicationDecorRatio(float ratio);

    /** Recupere le ratio a appliquer sur l'affichage des objets graphiques pour respecter la coherence de l'ensemble
      * @return float, flottant, le ratio que l'on veut appliquer sur l'affichage des objets graphiques
      */
    float getAbeApplicationDecorRatio();

    /** Recupere les informations du systeme d'exploitation en cours
      * @see AbulEduApplicationV1::os()
      * @return QString, chaine de caractere, la chaine de caractere correspondante au systeme d'exploitation
      */
    QString getAbeApplicationThisOS() {return os();}

    /** Donne un pointeur sur le networkaccessmanager "global"
      * @see AbulEduNetworkAccessManagerV1
      * @return AbulEduNetworkAccessManagerV1*, un pointeur sur le networkaccessmanager "global"
      */
    AbulEduNetworkAccessManagerV1* getAbeNetworkAccessManager();

    /** Donne un pointeur sur l'identité de l'utilisateur, un logiciel ne pouvant être utilisé que par une seule personne
      * @see AbulEduIdentitesV1
      * @return AbulEduIdentitesV1*
      */
    AbulEduIdentitesV1* getAbeIdentite();

    /** Retourne le chemin du répertoire utilisateur sans le / final
      *
      */
    QString getAbeHomepath();

    /** Emet le signal de fin de download
      * @see AbulEduApplicationV1::signalDownloaded(QString s)
      * @param QString s, chaine de caractere, !!
      * @brief methode virtuelle
      * @return void
      */
    virtual void sendSignalDownloaded(QString s);

    /** Retourne un numero unique, permet de remonter des statistiques d'utilisation anonymes
      * @return QString, chaine de caractere, un identifiant unique
      * @todo Convention de nommage accesseur, a modifier en getUniqueUID()
      */
    QString uniqueUID();

    /** Retourne le nom de l'utilisateur actuellement loggué (ouverture de session de l'OS, pas le login SSO)
      * @return QString, chaine de caractere, le login de l'utilisateur
      */
    QString getAbeUserLogin();

    /** Active ou désactive le clavier virtuel, pratique si on passe une appli sur le TBI et qu'on souhaite
      * bénéficier du clavier virtuel temporairement
      */
    void setAbeApplicationVirtualKeyboard(bool enabled);

private slots:
    /** Gestion de la requete reseau pour voir s'il existe une mise a jour de l'application
      * @param QNetworkReply*, un pointeur sur QNetworkReply
      * @return void
      */
    void slotOnlineUpdateRequest(QNetworkReply*);

    /** Gestion de l'erreur reseau
      * @see enum QNetworkReply::NetworkError
      * @param QNetworkReply::NetworkError code, le code erreur
      * @return void
      */
    void slotReplyError(QNetworkReply::NetworkError code);

private:
    /** Le gestionnaire de connexion reseau
      * @see AbulEduNetworkAccessManagerV1
      * @brief m_nam, pointeur sur un objet de type AbulEduNetworkAccessManagerV1
      */
    AbulEduNetworkAccessManagerV1*   m_nam;
    /** Le reply
      * @see QNetworkReply
      * @brief m_networkReply, un pointeur sur un objet de type QNetworkReply
      */
    QNetworkReply*                   m_networkReply;
    /** Espace memoire partagee pour test si une mise a jour est disponible (interne)
      * @see QSharedMemory
      * @brief m_sharedMemory, un objet de type QSharedmemory
      */
    QSharedMemory                    m_sharedMemory;
    /** Passage du module en mode debug ou pas
      * @brief m_localDebug, int
      */
    int                              m_localDebug;
    /** Nom de l'application
      * @brief m_applicationLongName, QString
      */
    QString                          m_applicationLongName;

    /** Langue à utiliser dans les interfaces, de type "fr" */
    QString                          m_interfaceLang;

    /** Facteur de mise a l'echelle
      * @brief Permet de changer la taille de tous les objets graphiques de maniere homogene
      * par rapport au dessin de depart
      * @brief m_decorRatio, float
      */
    float                            m_decorRatio;
    /** booleen qui evite d'ouvrir plusieurs fenetres de navigateur (i.e. si on a un vieux logiciel)
      * @param m_browserLaunched bool
      */
    bool                             m_browserLaunched;
    /** Format du package linux
      * @brief m_linuxPackageFormat, QString
      */
    QString                          m_linuxPackageFormat;

    /** Distribution linux
      * @brief m_linuxDistrib, QString
      */
    QString                          m_linuxDistrib;

    /** Répertoire home de l'utilisateur, pour eviter de tout mettre dans le profil itinérant tout windows
      * et pointer directement vers le même dossier que si on est en client lourd linux, ça permet d'avoir
      * un espace réellement commun quel que soit l'os sur lequel on travaile
      * @brief m_homepath, QString
      */
    QString                          m_homepath;
    void p_setUserName();

    /** Le login de l'utilisateur */
    QString                          m_username;

    /** Permet de détacher la zone de mémoire partagée lors de l'appel au destructeur
      * @return void
      */
    void detach();

    /** Permet de charger les plugins
      * @return void
      */
    void loadPlugins();

    /** Retourne le nom du format de paquets utilises par ce linux
      * @return QString, chaine de caractere, le nom du format de paquets utilises par ce linux
      */
    QString linuxPackageFormat();

    /** Retourne le nom de la distribution linux
      * @return QString, chaine de caractere, le nom de la distribution linux
      * @todo Convention de nommage accesseur, a modifier en getLinuxDistrib()
      */
    QString linuxDistrib();


    void linuxAnalyse();

    /** Retourne le nom du systeme d'exploitation
      * @return QString, chaine de caractere, la chaine de caractere correspondante au systeme d'exploitation
      * @todo Convention de nommage accesseur, a modifier en getOs()
      */
    QString os();

    /** Retourne l'age de du fichier executable de l'application (en nombre de jours)
      * @return int, entier, l'age du fichier
      * @todo Convention de nommage accesseur, a modifier en getExecAge()
      */
    int execAge();

    /** L'identité de la personne qui utilise l'application
      * @see AbulEduIdentitesV1
      * @brief m_identite, pointeur sur un objet de type AbulEduIdentitesV1
      */
    AbulEduIdentitesV1*   m_identite;

    /** Le clavier virtuel
      * @see AbulEduVirtualKeyboardV1
      * @brief m_abuleduVirtualKeyboard, pointeur sur un objet de type AbulEduVirtualKeyboardV1
      */
    AbulEduVirtualKeyboardV1 *m_abuleduVirtualKeyboard;

signals:
    /** Un signal pour indiquer que l'application est prête */
    void signalApplicationReady();

    /** Lancement du signal par le plugin
      * @see AbulEduApplicationV1::sendSignalDownloaded()
      * @brief Permet d'accrocher un slot de traitement via la méthode publique sendSignalDownloaded()
      * lorsque le telechargement est termine... (par ex. rafraichir la liste d'une combobox)
      */
    void signalDownloaded(QString s);

    /**
      * L'application à un message a faire passer, merci de cacher un splashscreen ou tout autre fenêtre qui
      * pourrait venir s'intercaller !
      */
    void signalApplicationMessageStart();

    /**
      * Fin du message a faire passer, merci de reafficher le splashscreen ou tout autre fenêtre
      */
    void signalApplicationMessageEnd();

};

/**
  * Astuce pour faire un sleep, pratique pour le splashscreen mais probablement a d'autres moments aussi
  * d'ou sa presence ici pour qu'elle soit accessible depuis n'importe quel endroit avec un simple
  * #include abuleduapplicationv1.h :)
  */
class AbulEduSleeperThread : public QThread
{
public:
    static void msleep(unsigned long msecs) {QThread::msleep(msecs);}
};

#endif //end ABULEDUAPPLICATIONV1_H
